#!/bin/bash

NEWAPPNAME=RAMS-2.1.0

if [ -z "$1" ]
then 
	OLDAPPNAME=RAMS-2.0.0
	echo "INFO: Beginning backing up old application by dafaulting to $OLDAPPNAME"
else
	OLDAPPNAME=$1
	echo "INFO: Beginning backing up old application $OLDAPPNAME"
fi

#See if Tomcat home directory, $CATALINA_HOME, is defined and if it exists
if [ -z ${CATALINA_HOME+x} ] 
then
   echo "INFO: \$CATALINA_HOME is not set - defaulting to /usr/share/apache-tomcat-7.0.57" 
   CATALINA_HOME="/usr/share/apache-tomcat-7.0.57"
else 
   echo "INFO: using \$CATALINA_HOME environment variable value $CATALINA_HOME" 
fi

if [ -d $CATALINA_HOME ]
then
   echo "INFO: \$CATALINA_HOME found"
else 
   echo "INFO: \$CATALINA_HOME not found at $CATALINA_HOME"
fi

cd $CATALINA_HOME
if [ ! -d $CATALINA_HOME/backup ]
then 
   echo "INFO: creating \$CATALINA_HOME/backup" 
   mkdir -p $CATALINA_HOME/backup
else 
   echo "INFO: \$CATALINA_HOME/backup found"
fi

if [ ! -d $CATALINA_HOME/deploy ]
then 
   echo "INFO: creating \$CATALINA_HOME/deploy" 
   mkdir -p $CATALINA_HOME/deploy
else 
   echo "INFO: \$CATALINA_HOME/deploy found"
fi

pid=$(ps x | grep "$CATALINA_HOME" | grep -v grep | cut -d ' ' -f 1)
if [ "${pid}" ]; then
	echo "INFO: stopping tomcat (pid=$pid) ..."
	sudo service tomcat7 stop
fi

echo "INFO: saving the RAMS-*.xml and RAMS-*.war files to $CATALINA_HOME/backup"
cp $CATALINA_HOME/conf/Catalina/localhost/RAMS-*.xml $CATALINA_HOME/backup/.
cp $CATALINA_HOME/webapps/RAMS-*.war $CATALINA_HOME/backup/.

echo "INFO: removing the RAMS-*.xml and RAMS-*.war files and the exploded war"
rm $CATALINA_HOME/conf/Catalina/localhost/RAMS-*.xml
rm $CATALINA_HOME/webapps/RAMS-*.war
rm -rf $CATALINA_HOME/webapps/RAMS-*

CODEC=commons-codec-1.3.jar
if [ -f $CATALINA_HOME/lib/$CODEC ]
then 
   echo "INFO: backing up $CODEC"
   mv $CATALINA_HOME/lib/$CODEC $CATALINA_HOME/backup/$CODEC
else 
   echo "WARNING: $CODEC not found in \$CATALINA_HOME/lib"
fi

rc=0
echo "INFO: verifying that the files are in the backup location and no longer in the deployed location"
if [ -f $CATALINA_HOME/conf/Catalina/localhost/$OLDAPPNAME.xml ]
then 
   echo "ERROR: $OLDAPPNAME.xml still exists in \$CATALINA_HOME/conf/Catalina/localhost directory"
   rc=1
fi
if [ -f $CATALINA_HOME/webapps/$OLDAPPNAME.war ]
then 
   echo "ERROR: $OLDAPPNAME.war still exists in \$CATALINA_HOME/webapps directory"
   rc=2
fi
if [ -d $CATALINA_HOME/webapps/$OLDAPPNAME ]
then 
   echo "ERROR: RAMS-* directory still exists in \$CATALINA_HOME/webapps directory"
   rc=3
fi
if [ ! -f $CATALINA_HOME/backup/$OLDAPPNAME.xml ]
then 
   echo "ERROR: $OLDAPPNAME.xml not found in \$CATALINA_HOME/backup directory"
   rc=4
fi
if [ ! -f $CATALINA_HOME/backup/$OLDAPPNAME.war ]
then 
   echo "ERROR: $OLDAPPNAME.war not found in \$CATALINA_HOME/backup directory"
   rc=5
fi
if [ ! -f $CATALINA_HOME/backup/$CODEC ]
then 
   echo "WARNING: $CODEC not found in \$CATALINA_HOME/backup directory"
   rc=6
fi

echo "INFO: Backup Done - exit code=$rc"
exit $rc
