#!/bin/bash


if [ -z "$1" ]
then 
	BASENAME=RAMS-2.1.0
	echo "INFO: Beginning deployment using default application name $BASENAME"
else
	BASENAME=$1
	echo "INFO: Beginning deployment of $BASENAME"
fi

#See if Tomcat home directory, $CATALINA_HOME, is defined and if it exists
if [ -z ${CATALINA_HOME+x} ] 
then
   echo "INFO: \$CATALINA_HOME is not set - defaulting to /usr/share/apache-tomcat-7.0.57" 
   CATALINA_HOME=/usr/share/apache-tomcat-7.0.57
else 
   echo "INFO: using \$CATALINA_HOME environment variable value $CATALINA_HOME"
fi

if [ -d $CATALINA_HOME ]
then
   echo "INFO: \$CATALINA_HOME found"
else 
   echo "INFO: \$CATALINA_HOME not found at $CATALINA_HOME"
fi

if [ ! -d $CATALINA_HOME/backup ]
then 
   echo "INFO: creating \$CATALINA_HOME/backup" 
   mkdir -p $CATALINA_HOME/backup
else 
   echo "INFO: \$CATALINA_HOME/backup found"
fi

if [ ! -d $CATALINA_HOME/deploy ]
then 
   echo "INFO: creating \$CATALINA_HOME/deploy" 
   mkdir -p $CATALINA_HOME/deploy
else 
   echo "INFO: \$CATALINA_HOME/deploy found"
fi

((ERR=0))
if [ -f "$CATALINA_HOME/deploy/$BASENAME.xml" ]
then 
   echo "INFO: $BASENAME.xml file found in \$CATALINA_HOME/deploy" 
else 
   echo "ERROR: no $BASENAME.xml file found in \$CATALINA_HOME/deploy"
   ((ERR++))
fi

if [ -f "$CATALINA_HOME/deploy/$BASENAME.war" ]
then 
   echo "INFO: $BASENAME.war file found in \$CATALINA_HOME/deploy" 
else 
   echo "ERROR: no $BASENAME.war file found in \$CATALINA_HOME/deploy"
   ((ERR++))
fi

if [ $ERR = 0 ]
then
   pid=$(ps x | grep "$CATALINA_HOME" | grep -v grep | cut -d ' ' -f 1)
   if [ "${pid}" ]; then
	  echo "INFO: stopping tomcat (pid=$pid) ..."
      sudo service tomcat7 stop
   fi

   echo "INFO: deploying the $BASENAME.xml and $BASENAME.war files"
   cp $CATALINA_HOME/deploy/$BASENAME.xml $CATALINA_HOME/conf/Catalina/localhost
   cp $CATALINA_HOME/deploy/$BASENAME.war $CATALINA_HOME/webapps
   chown apache:apache $CATALINA_HOME/conf/Catalina/localhost/$BASENAME.xml
   chmod 644 $CATALINA_HOME/conf/Catalina/localhost/$BASENAME.xml
   chown apache:apache $CATALINA_HOME/webapps/$BASENAME.war
   chmod 774 $CATALINA_HOME/webapps/$BASENAME.war
 
   CODEC=commons-codec-1.9.jar
   echo "INFO: deploying the $CODEC files"
   cp $CATALINA_HOME/deploy/$CODEC $CATALINA_HOME/lib/$CODEC
   chmod 644 $CATALINA_HOME/lib/$CODEC
   chown apache:apache $CATALINA_HOME/lib/$CODEC 
   echo "INFO: verifying deployment"
else
   echo "ERROR: error found - nothing deployed"
   exit $ERR
fi   

rc=0
echo "INFO: verifying that the files are in the backup location and no longer in the deployed location"
if [ ! -f $CATALINA_HOME/conf/Catalina/localhost/$BASENAME.xml ]
then 
   echo "ERROR: $BASENAME.xml does not exist in \$CATALINA_HOME/conf/Catalina/localhost directory"
   rc=1
fi
if [ ! -f $CATALINA_HOME/webapps/$BASENAME.war ]
then 
   echo "ERROR: $BASENAME.war does not exist in \$CATALINA_HOME/webapps directory"
   rc=2
fi
if [ ! -f $CATALINA_HOME/lib/$CODEC ]
then 
   echo "ERROR: $CODEC not found in \$CATALINA_HOME/lib directory"
   rc=3
fi

echo "INFO: Deployment Done - exit code=$rc"
exit $rc
