package gov.va.rams.entities;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;

import java.util.TimeZone;
import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlElement;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import gov.va.rams.service.ServiceException;
import java.util.regex.Pattern;

/**
 *
 * @author Brian
 */
@Entity
@Table(name = "Studies", uniqueConstraints = { @UniqueConstraint(columnNames = { "AssignedID" }) })
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
@NamedQueries({ @NamedQuery(name = "Study.findByAssignedID", query = "SELECT s FROM Study s WHERE s.assignedID = :assignedID"),
        @NamedQuery(name = "Study.findByCreator", query = "SELECT ss.study FROM StudyStatus ss WHERE ss.stageCounter = 1 AND ss.createdBy = :creator"),
        @NamedQuery(name = "Study.findAll", query = "SELECT s FROM Study s") })
public class Study implements Serializable {

    static final Logger logz = LoggerFactory.getLogger(Study.class);

    private static final long serialVersionUID = 1L;
    @Id
    @Basic(optional = false)
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column
    @XmlElement
    private Integer studiesID;
    @Basic(optional = false)
    @NotNull
    @Size(min = 1, max = 15)
    @Column(nullable = false, length = 15)
    @XmlElement
    private String assignedID;
    @Basic(optional = false)
    @NotNull
    @Size(min = 1, max = 200)
    @Column(nullable = false, length = 200)
    @XmlElement
    private String title;
    @Basic
    @Column
    @XmlElement
    private Integer spListItem;
    @Basic
    @Column
    @Temporal(TemporalType.TIMESTAMP)
    @XmlElement
    private Date synchronizedTimestamp;
    @JoinColumn(name = "FacilitiesID", referencedColumnName = "FacilitiesID", nullable = false)
    @ManyToOne(optional = false)
    @XmlElement
    private Facility facility;
    @JoinColumn(name = "ParentStudiesID", referencedColumnName = "StudiesID")
    @ManyToOne(fetch = FetchType.LAZY)
    private Study parentStudy;
    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER, mappedBy = "study")
    @OrderBy("stageCounter")
    @XmlElement
    private Collection<StudyStatus> studyStatusCollection;
    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER, mappedBy = "study")
    @XmlElement
    private Collection<Study_Members> studyMembersCollection;
    @Basic(optional = false)
    @NotNull
    @Column(nullable = false)
    @XmlElement
    private Integer formVersionsID;

    public Study() {
    }

    public Integer getStudiesID() {
        return studiesID;
    }

    public void setStudiesID(Integer studiesID) {
        this.studiesID = studiesID;
    }

    public String getAssignedID() {
        return assignedID;
    }

    public void setAssignedID(String assignedID) {
        Pattern p = Pattern.compile("^[0-9]{14}[HA]");//nospaces; no specialcharacters; 15 char long = 14 numbers followed by H or A
        if (assignedID.equals("") | (assignedID.length() == 0) | !p.matcher(assignedID).find()) {
            logz.debug("Invalid assignedID");
            throw ServiceException.notFound();
        }
        this.assignedID = assignedID;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public Date getSynchronizedTimestamp() {
        return synchronizedTimestamp;
    }

    public void setSynchronizedTimestamp(Date synchronizedTimestamp) {
        this.synchronizedTimestamp = synchronizedTimestamp;
    }

    public Facility getFacility() {
        return facility;
    }

    public void setFacility(Facility facility) {
        this.facility = facility;
    }

    @XmlTransient
    public Study getParentStudy() {
        return parentStudy;
    }

    public void setParentStudy(Study parentStudy) {
        this.parentStudy = parentStudy;
    }

    public Collection<StudyStatus> getStudyStatusCollection() {
        return studyStatusCollection;
    }

    public void setStudyStatusCollection(Collection<StudyStatus> studyStatusCollection) {
        this.studyStatusCollection = studyStatusCollection;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (studiesID != null ? studiesID.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Study)) {
            return false;
        }
        Study other = (Study) object;
        if ((this.studiesID == null && other.studiesID != null) || (this.studiesID != null && !this.studiesID.equals(other.studiesID))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "gov.va.rams.Study[ studiesID=" + studiesID + " ]";
    }

    public Integer getFormVersionsID() {
        return formVersionsID;
    }

    public void setFormVersionsID(Integer formVersionsID) {
        this.formVersionsID = formVersionsID;
    }

    public String getYear() {
        Date start = getStudyStatusCollection().iterator().next().getCreatedAt();
        Calendar c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
        c.setTimeInMillis(start.getTime());
        return String.valueOf(c.get(Calendar.YEAR));
    }

    public Collection<Study_Members> getStudyMembersCollection() {
        return studyMembersCollection;
    }

    public void setStudyMembersCollection(Collection<Study_Members> studyMembersCollection) {
        this.studyMembersCollection = studyMembersCollection;
    }

    public Integer getSPListItem() {
        return spListItem;
    }

    public void setSPListItem(Integer spListItem) {
        this.spListItem = spListItem;
    }

    public boolean isUnclaimed() {
        boolean unclaimed = true;
        for (Study_Members membership : getStudyMembersCollection()) {
            if (membership.getUser() != null) {
                unclaimed = false;
                break;
            }
        }
        return unclaimed;
    }
}
