package gov.va.rams.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import gov.va.rams.docx4j.DocXFormGenerator;
import gov.va.rams.entities.Comment;
import gov.va.rams.entities.ContainedForm;
import gov.va.rams.entities.Facility;
import gov.va.rams.entities.Form;
import gov.va.rams.entities.FormQuestion;
import gov.va.rams.entities.FormVersion;
import gov.va.rams.entities.QuestionResponse;
import gov.va.rams.entities.Study;
import gov.va.rams.entities.StudyState;
import gov.va.rams.entities.StudyStatus;
import gov.va.rams.entities.User;
import gov.va.rams.entities.Study_Members;
import gov.va.rams.service.elements.FormValidationReport;
import gov.va.rams.service.elements.QuestionResponseUpdates;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.NoResultException;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.DELETE;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.GenericEntity;

/**
 *
 * @author Brian
 */
@Stateless
@Consumes(MediaType.APPLICATION_JSON)
@Produces(MediaType.APPLICATION_JSON)
@Path("")
public class ServiceREST extends Authenticated {
    static final Logger logz = LoggerFactory.getLogger(ServiceREST.class);

    @GET
    @Path("forms")
    public Response getRootForms() {
        List<FormVersion> lfv = em.createNamedQuery("FormVersion.findAll", FormVersion.class).getResultList();
        return Response.ok(new GenericEntity<List<FormVersion>>(lfv) {}).build();
    }

    @GET
    @Path("forms/{version}")
    public Response getRootForm(@PathParam("version") Integer version) {
        permitCaching();
        return Response.ok(em.find(FormVersion.class, version).getRootForm()).build();
    }

    @GET
    @Path("user")
    public Response getUsers() {
        List<User> lu = em.createNamedQuery("User.findAll", User.class).getResultList();
        return Response.ok(new GenericEntity<List<User>>(lu) {}).build();
    }

    @POST
    @Path("user")
    @Consumes
    public Response postUser(@QueryParam("loginName") String loginName) {
        return Response.ok(loginName != null ? super.getOrUpdateOrCreateUser(loginName) : super.getCurrentUser()).build();
    }

    @POST
    @Path("user/{id}")
    @Consumes
    public User postUserById(@PathParam("id") Integer id) {
        return super.getOrUpdateUser(id);
    }

    @GET
    @Path("facility")
    public Response getFacilities() {
        List<Facility> lf = em.createNamedQuery("Facility.findAll", Facility.class).getResultList();
        return Response.ok(new GenericEntity<List<Facility>>(lf) {}).build();
    }

    @POST
    @Path("study/formVersionsID/{formVersionsID}")
    public Response addStudy(Study study, @PathParam("formVersionsID") Integer formVersionsID) {
        study.setFormVersionsID(formVersionsID);
        return addStudy(study);
    }

    @POST
    @Path("study")
    public Response addStudy(Study study) {
        final Date timestamp = new Date();
        final User user = getCurrentUser();
        StudyStatus ss = new StudyStatus();
        ss.setCreatedAt(timestamp);
        ss.setCreatedBy(user);
        ss.setUpdatedAt(timestamp);
        ss.setUpdatedBy(user);
        if (study.getStudyMembersCollection() == null || study.getStudyMembersCollection().isEmpty()) {
            Study_Members autoAdd = new Study_Members();
            autoAdd.setActiveMember(true);
            autoAdd.setCreatedAt(timestamp);
            autoAdd.setExplaination("Implicit membership due to user initiating creation of the study application.");
            autoAdd.setMemberRole("Author");
            autoAdd.setNotifications(true);
            autoAdd.setTeamMember(true);
            autoAdd.setUser(user);
            autoAdd.setWritePermissions(true);
            study.setStudyMembersCollection(Collections.singletonList(autoAdd));
        }
        for (Study_Members membership : study.getStudyMembersCollection())
            membership.setStudy(study);
        ss.setStageCounter(1);
        ss.setStudyState(getStudyState("draft"));
        ss.setStudy(study);
        study.setFacility(em.getReference(Facility.class, study.getFacility().getFacilitiesID()));
        study.setStudyStatusCollection(Collections.singletonList(ss));
        em.persist(study);
        em.flush();
        em.refresh(study);
        sp.addStudy(study);
        return Response.ok(study).build();
    }

    @GET
    @Path("study/{id}")
    public Response retrieveStudy(@PathParam("id") String id) {
        final Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        authorize(getCurrentUser(), Access.VIEW, s);

        return Response.ok(s).build();
    }

    @GET
    @Path("study")
    public Response retrieveStudiesForUser(@PathParam("id") String id) {
        User u = getCurrentUser();
        Collection<Study> studies = new HashSet<>(u.getStudyMembersCollection().size());

        for (Study_Members membership : u.getStudyMembersCollection())
            if (membership.isActiveMember() && membership.isTeamMember())
                studies.add(membership.getStudy());

        studies.addAll(em.createNamedQuery("Study.findByCreator", Study.class).setParameter("creator", u).getResultList());

        return Response.ok(new GenericEntity<Collection<Study>>(studies) {}).build();
    }

    @GET
    @Path("unclaimed")
    public Response retrieveUnclaimedStudies(@PathParam("id") String id) {
        Collection<Study> unclaimedStudies = new ArrayList<>();
        for (Study s : em.createNamedQuery("Study.findAll", Study.class).getResultList()) {
            if (s.isUnclaimed())
                unclaimedStudies.add(s);
        }
        return Response.ok(new GenericEntity<Collection<Study>>(unclaimedStudies) {}).build();
    }

    @GET
    @Path("study/{id}/SPDocumentSite")
    @Produces(MediaType.TEXT_PLAIN)
    public String getStudySPDocumentSite(@PathParam("id") String id) {
        final Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        return sp.getDocumentSiteUrl(s);
    }

    @POST
    @Path("study/{id}/SPDocumentSite")
    @Consumes
    public void updateSPDocumentSitePermissions(@PathParam("id") String id) {
        final Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        sp.updateFolderPermissions(s);
    }

    @GET
    @Path("study/{id}/parentstudy")
    public Response retrieveStudyParent(@PathParam("id") String id) {
        final Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        final User user = getCurrentUser();
        authorize(user, Access.VIEW, s);
        authorize(user, Access.VIEW, s.getParentStudy());

        return Response.ok(s.getParentStudy()).build();
    }

    @POST
    @Path("study/{id}/parentstudy/{pid}")
    public void setStudyParent(@PathParam("id") String id, @PathParam("pid") String pid) {
        final Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        final Study p = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", pid).getSingleResult();
        final User user = getCurrentUser();
        authorize(user, Access.MODIFY, s);
        authorize(user, Access.VIEW, p);

        s.setParentStudy(p);
    }

    @GET
    @Path("study/{id}/members")
    public Response retrieveMembers(@PathParam("id") String id) {
        final Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        authorize(getCurrentUser(), Access.VIEW, s);

        return Response.ok(new GenericEntity<Collection<Study_Members>>(s.getStudyMembersCollection()) {}).build();
    }

    // Compatibility (now deprecated in favor of the following endpoint)
    @POST
    @Path("study/{id}/members/{uid}")
    public void setMembers(Study_Members sm, @PathParam("id") String id, @PathParam("uid") Integer uid) {
        sm.setStudy_MembersID(null);
        sm.setUser(em.getReference(User.class, uid));
        addOrUpdateMember(id, sm);
    }

    @POST
    @Path("study/{id}/members")
    public void addOrUpdateMember(@PathParam("id") String id, Study_Members sm) {
        final Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        authorize(getCurrentUser(), Access.MODIFY, s);

        if (sm.getUser() != null)
            sm.setUser(em.find(User.class, sm.getUser().getUsersID()));
        if (sm.getStudy_MembersID() == null) {
            sm.setStudy(s);
            sm.setCreatedAt(new Date());
            s.getStudyMembersCollection().add(sm);
            em.persist(sm);
            sp.updateFolderPermissions(s);
            return;
        } else {
            for (Study_Members search : s.getStudyMembersCollection()) {
                if (sm.equals(search)) {
                    search.setActiveMember(sm.isActiveMember());
                    search.setCoiRequired(sm.isCoiRequired());
                    search.setExplaination(sm.getExplaination());
                    search.setMemberRole(sm.getMemberRole());
                    search.setNotifications(sm.isNotifications());
                    search.setRemovedAt(sm.getRemovedAt());
                    search.setTeamMember(sm.isTeamMember());
                    search.setUser(sm.getUser());
                    search.setWritePermissions(sm.isWritePermissions());
                    sp.updateFolderPermissions(s);
                    return;
                }
            }
        }
        throw ServiceException.notFound();
    }

    @POST
    @Path("study/{id}/aomembers")
    public void addStudyMember(@PathParam("id") String id, Study_Members sm) {
        final Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();

        if (sm.getUser() != null)
            sm.setUser(em.find(User.class, sm.getUser().getUsersID()));
        if (sm.getStudy_MembersID() == null) {
            sm.setStudy(s);
            sm.setCreatedAt(new Date());
            s.getStudyMembersCollection().add(sm);
            em.persist(sm);
            sp.updateFolderPermissions(s);
            return;
        } else {
            for (Study_Members search : s.getStudyMembersCollection()) {
                if (sm.equals(search)) {
                    search.setActiveMember(sm.isActiveMember());
                    search.setCoiRequired(sm.isCoiRequired());
                    search.setExplaination(sm.getExplaination());
                    search.setMemberRole(sm.getMemberRole());
                    search.setNotifications(sm.isNotifications());
                    search.setRemovedAt(sm.getRemovedAt());
                    search.setTeamMember(sm.isTeamMember());
                    search.setUser(sm.getUser());
                    search.setWritePermissions(sm.isWritePermissions());
                    sp.updateFolderPermissions(s);
                    return;
                }
            }
        }
        throw ServiceException.notFound();
    }

    @POST
    @Path("study/{id}/removemembers")
    public void removeStudyMember(@PathParam("id") String id, Study_Members sm) {
        final Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        Study_Members mem = em.find(Study_Members.class, sm.getStudy_MembersID());
        em.remove(mem);
        sp.updateFolderPermissions(s);
        return;
    }

    @POST
    @Consumes
    @Path("study/{id}/stage/{state}")
    public Response addStage(@PathParam("id") String id, @PathParam("state") String state) {
        final Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        final User user = getCurrentUser();
        authorize(user, Access.ADVANCE, s);

        final Date timestamp = new Date();
        final StudyStatus ss = new StudyStatus();
        ss.setStudy(s);
        s.getStudyStatusCollection().add(ss);
        ss.setStageCounter(s.getStudyStatusCollection().size());
        ss.setStudyState(getStudyState(state));
        ss.setCreatedAt(timestamp);
        ss.setUpdatedAt(timestamp);
        ss.setCreatedBy(user);
        ss.setUpdatedBy(user);
        em.flush();
        em.refresh(s);
        sp.addStageToStudy(ss);
        return Response.ok(s).build();
    }

    // XXX Adding states here needs to go away.  All states should be defined when the database is initialized.
    private StudyState getStudyState(String name) {
        List<StudyState> results = em.createNamedQuery("StudyState.findByName", StudyState.class).setParameter("name", name).getResultList();
        if (results.isEmpty()) {
            StudyState ss = new StudyState();
            ss.setName(name);
            em.persist(ss);
            return ss;
        } else {
            return results.get(0);
        }
    }

    @GET
    @Path("study/{id}/responses/{stage}")
    public Response retrieveStudyResponsesByStage(@PathParam("id") String id, @PathParam("stage") Integer stage) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        authorize(getCurrentUser(), Access.VIEW, s);

        for (StudyStatus ss : s.getStudyStatusCollection())
            if (ss.getStageCounter() == stage)
                return Response.ok(new GenericEntity<Collection<QuestionResponse>>(ss.getQuestionResponseCollection()) {}).build();
        throw ServiceException.notFound();
    }

    @EJB
    private ResponseValidator responseValidator;

    @GET
    @Path("study/{id}/responses/{stage}/rollup")
    public Response retrieveStudyResponsesRollup(@PathParam("id") String id, @PathParam("stage") Integer lastStage) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        authorize(getCurrentUser(), Access.VIEW, s);

        List<QuestionResponse> responsesByStage = new ArrayList<>();
        QuestionResponseUpdates updates = new QuestionResponseUpdates();
        StudyStatus status = null;
        for (StudyStatus ss : s.getStudyStatusCollection()) {
            responsesByStage.addAll(ss.getQuestionResponseCollection());
            if (ss.getStageCounter() == lastStage) {
                updates.setUpdateTimestamp(ss.getUpdatedAt());
                status = ss;
                break;
            }
        }
        if (status == null)
            throw ServiceException.notFound();
        HashMap<QuestionResponse.Key, QuestionResponse> rollup = QuestionResponse.rollup(responsesByStage);
        updates.setResponses(rollup.values());
        updates.setFormValidationReport(responseValidator.getFormValidationReport(status));
        if (updates.getFormValidationReport() == null)
            updates.setFormValidationReport(responseValidator.updateFormValidationReport(status, rollup, false));

        return Response.ok(updates).build();
    }

    @GET
    @Path("study/{id}/responses/{stage}/validation")
    public FormValidationReport performStudyFormValidation(@PathParam("id") String id, @PathParam("stage") Integer lastStage) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        authorize(getCurrentUser(), Access.VIEW, s);

        FormValidationReport report = null;
        for (StudyStatus ss : s.getStudyStatusCollection()) {
            if (ss.getStageCounter() == lastStage) {
                report = responseValidator.getFormValidationReport(ss);
                break;
            }
        }
        if (report == null) {
            List<QuestionResponse> responsesByStage = new ArrayList<>();
            StudyStatus status = null;
            for (StudyStatus ss : s.getStudyStatusCollection()) {
                responsesByStage.addAll(ss.getQuestionResponseCollection());
                if (ss.getStageCounter() == lastStage) {
                    status = ss;
                    break;
                }
            }
            if (status == null)
                throw ServiceException.notFound();
            report = responseValidator.updateFormValidationReport(status, QuestionResponse.rollup(responsesByStage), false);
        }
        return report;
    }

    @POST
    @Consumes
    @Path("study/{id}/responses/{stage}/cleanup")
    public void performResponseCleanup(@PathParam("id") String id, @PathParam("stage") Integer lastStage) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        authorize(getCurrentUser(), Access.MODIFY, s);
        StudyStatus latest = ensureCurrentStage(s, lastStage);

        List<QuestionResponse> responsesByStage = new ArrayList<>();
        for (StudyStatus ss : s.getStudyStatusCollection())
            responsesByStage.addAll(ss.getQuestionResponseCollection());
        responseValidator.updateFormValidationReport(latest, QuestionResponse.rollup(responsesByStage), true);
    }

    private static StudyStatus ensureCurrentStage(Study s, Integer lastStage) throws ServiceException {
        StudyStatus latest = null;
        for (StudyStatus ss : s.getStudyStatusCollection()) {
            if (ss.getStageCounter() == lastStage)
                latest = ss;
            else if (ss.getStageCounter() > lastStage)
                throw ServiceException.forbidden();
        }
        if (latest == null)
            throw ServiceException.notFound();
        return latest;
    }

    @EJB
    private StudyStatusNotifier studyStatusNotifier;
    @EJB // We use this to separate out the transactional contexts into appropriate lifetimes.
    private QuestionResponseUpdater questionResponseUpdater;

    @POST
    @Path("study/{id}/responses/{stage}")
    @TransactionAttribute(TransactionAttributeType.NEVER)
    public void setStudyResponses(@PathParam("id") String id, @PathParam("stage") Integer stage, Collection<QuestionResponse> responses) {
        studyStatusNotifier.notifyUpdate(questionResponseUpdater.setStudyResponses(id, stage, responses, getCurrentUser()));
    }

    // IRB Administrators can see all comments, but study members and reviewers can only view IRB Administrator comments or their own.
    private Collection<Comment> filterComments(Study study, Collection<Comment> commentCollection) {
        if (isAdministrator(getCurrentUser(), study))
            return commentCollection;
        if (!isReviewer(getCurrentUser(), study))
            authorize(getCurrentUser(), Access.COMMENT, study);

        ArrayList<Comment> filtered = new ArrayList<>();
        for (Comment c : commentCollection)
            if (getCurrentUser().equals(c.getUser()) || isAdministrator(c.getUser(), study))
                filtered.add(c);
        return filtered;
    }

    @GET
    @Path("study/{id}/comments/{stage}")
    public Response retrieveStudyCommentsByStage(@PathParam("id") String id, @PathParam("stage") Integer stage) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();

        for (StudyStatus ss : s.getStudyStatusCollection())
            if (ss.getStageCounter() == stage)
                return Response.ok(new GenericEntity<Collection<Comment>>(filterComments(s, ss.getCommentCollection())) {}).build();
        throw ServiceException.notFound();
    }

    @GET
    @Path("study/{id}/comments/{stage}/rollup")
    public Response retrieveStudyCommentsRollup(@PathParam("id") String id, @PathParam("stage") Integer stage) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();

        ArrayList<Comment> comments = new ArrayList<>();
        for (StudyStatus ss : s.getStudyStatusCollection())
            if (ss.getStageCounter() <= stage)
                comments.addAll(ss.getCommentCollection());
            else
                break;
        return Response.ok(new GenericEntity<Collection<Comment>>(filterComments(s, comments)) {}).build();
    }

    @DELETE
    @Path("study/{id}/comments/{stage}/{commentsId:[0-9]+}")
    public void removeStudyCommentFromStage(@PathParam("id") String id, @PathParam("stage") Integer stage, @PathParam("commentsId") Integer commentsId) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        User u = getCurrentUser();
        authorize(u, Access.COMMENT, s);
        StudyStatus latest = ensureCurrentStage(s, stage);

        for (Comment c : latest.getCommentCollection()) {
            if (c.getCommentsID().equals(commentsId)) {
                if (!c.getUser().equals(u))
                    throw ServiceException.notFound();
                em.remove(c);
                latest.setUpdatedAt(new Date());
                latest.setUpdatedBy(u);
                return;
            }
        }
    }

    @POST
    @Path("study/{id}/comments/{stage}/{commentsId:[0-9]+}")
    public void modifyStudyCommentInStage(@PathParam("id") String id, @PathParam("stage") Integer stage, @PathParam("commentsId") Integer commentsId, Comment comment) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        User u = getCurrentUser();
        authorize(u, Access.COMMENT, s);
        StudyStatus latest = ensureCurrentStage(s, stage);

        for (Comment c : latest.getCommentCollection()) {
            if (c.getCommentsID().equals(commentsId)) {
                if (!c.getUser().equals(u))
                    throw ServiceException.notFound();
                Date now = new Date();
                c.setAnswersID(comment.getAnswersID());
                c.setCreatedAt(now);
                c.setFormsID(comment.getFormsID());
                c.setHyperText(comment.getHyperText());
                c.setPublished(comment.getPublished());
                c.setQuestionsID(comment.getQuestionsID());
                c.setRedFlagged(comment.getRedFlagged());
                latest.setUpdatedAt(now);
                latest.setUpdatedBy(u);
                return;
            }
        }
        throw ServiceException.notFound();
    }

    @POST
    @Path("study/{id}/comments/{stage}")
    public void addStudyCommentsToStage(@PathParam("id") String id, @PathParam("stage") Integer stage, Collection<Comment> comments) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        User u = getCurrentUser();
        authorize(u, Access.COMMENT, s);
        StudyStatus latest = ensureCurrentStage(s, stage);

        Date now = new Date();
        for (Comment c : comments) {
            c.setCommentsID(null);
            c.setCreatedAt(now);
            c.setUser(u);
            c.setStudyStatus(latest);
            latest.getCommentCollection().add(c);
        }
        latest.setUpdatedAt(now);
        latest.setUpdatedBy(u);
    }

    @POST
    @Path("study/{id}/responses/{stage}/feed")
    public Response retrieveStudyResponseFeed(@PathParam("id") String id, @PathParam("stage") Integer stage, QuestionResponseUpdates request, @QueryParam("form") Integer formsID)
            throws InterruptedException {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        User u = getCurrentUser();
        authorize(u, Access.VIEW, s);

        try {
            Form f = em.find(Form.class, formsID);
            studyFormLock.renew(s, f, u);
        } catch (NoResultException ignoreBadLocking) {
        }
        for (StudyStatus ss : s.getStudyStatusCollection()) {
            if (ss.getStageCounter() == stage) {
                QuestionResponseUpdates response = new QuestionResponseUpdates();
                response.setResponses(new LinkedList<QuestionResponse>());
                if (!ss.getUpdatedAt().after(request.getUpdateTimestamp())) {
                    // We haven't gotten updates, so let's sleep on it.
                    if (studyStatusNotifier.awaitUpdate(ss, request.getUpdateTimestamp()))
                        ss = questionResponseUpdater.getUpdate(ss);
                }
                response.setUpdateTimestamp(ss.getUpdatedAt());
                for (QuestionResponse qr : ss.getQuestionResponseCollection()) {
                    if (qr.getUpdatedAt().after(request.getUpdateTimestamp()))
                        response.getResponses().add(qr);
                }
                response.setFormValidationReport(responseValidator.getFormValidationReport(ss)); // Must be in the if it changed since the call to GET .../rollup!
                return Response.ok(response).build();
            }
        }
        throw ServiceException.notFound();
    }

    @EJB
    private StudyFormLock studyFormLock;

    @PUT
    @Path("study/{id}/lock/{formid}")
    public void acquireLock(@PathParam("id") String studyId, @PathParam("formid") Integer formId) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", studyId).getSingleResult();
        User u = getCurrentUser();
        authorize(u, Access.VIEW, s);
        Form f = em.find(Form.class, formId);

        if (!studyFormLock.acquire(s, f, u))
            throw ServiceException.forbidden();
    }

    @DELETE
    @Path("study/{id}/lock/{formid}")
    public void releaseLock(@PathParam("id") String studyId, @PathParam("formid") Integer formId) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", studyId).getSingleResult();
        User u = getCurrentUser();
        authorize(u, Access.VIEW, s);
        Form f = em.find(Form.class, formId);

        if (!studyFormLock.release(s, f, u))
            throw ServiceException.forbidden();
    }

    @DELETE
    @Path("study/{id}/lock_a/{formid}")
    public void adminReleaseLock(@PathParam("id") String studyId, @PathParam("formid") Integer formId) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", studyId).getSingleResult();
        User u = getCurrentUser();
        authorize(u, Access.VIEW, s);
        Form f = em.find(Form.class, formId);

        if (studyFormLock.administrativeRelease(s, f, u)) {
            logz.debug("\nLock Released\n");
        } else {
            logz.debug("\nLock Not Released\n");
            throw ServiceException.forbidden();
        }
    }

    @POST
    @Path("study/{id}")
    public void updateStudy(@PathParam("id") String studyId, Study changed) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", studyId).getSingleResult();
        authorize(getCurrentUser(), Access.MODIFY, s);

        s.setTitle(changed.getTitle());
    }

    //temporary for debugging
    @GET
    @Produces(MediaType.TEXT_PLAIN)
    @Path("queryLock")
    public String queryLock() {
        return studyFormLock.inspect();
    }

    @GET
    @Path("study/{id}/documentGeneration/{stage}/{which}")
    public Response generateDocument(@PathParam("id") String id, @PathParam("stage") Integer lastStage, @PathParam("which") Integer sectionNumber,
            @QueryParam("approved") @DefaultValue("false") boolean approved) throws IOException {
        logz.debug("service found");
        logz.debug("Section number = " + sectionNumber);
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        User u = getCurrentUser();
        authorize(u, Access.DOCGEN, s);

        Form root = em.find(FormVersion.class, s.getFormVersionsID()).getRootForm();
        Form mySection = null;
        int hipaaSectionNumber = -1;
        logz.debug("Root Form Narrative = " + root.getNarrative());
        logz.debug("Contained Form Collection Size = " + root.getContainedFormCollection().size());

        Collection<ContainedForm> cfCollection = root.getContainedFormCollection();
        Iterator<ContainedForm> cfCollectionIter = cfCollection.iterator();
        while (cfCollectionIter.hasNext()) {
            ContainedForm cf = cfCollectionIter.next();
            logz.debug("Contain Form Forms Id = " + cf.getContainedFormsID() + ", sequence position = " + cf.getFormSequencePosition());
            if (cf.getContainedForm().getTitle().toLowerCase().trim().contains("hipaa")) {
                hipaaSectionNumber = cf.getFormSequencePosition();
                logz.debug("HIPAA Section Sequence number = " + hipaaSectionNumber);
            }
            if (cf.getFormSequencePosition() == sectionNumber) {
                mySection = cf.getContainedForm();
                logz.debug("Inside contained form loop - section found");
            }

        }

        if (mySection == null) {
            logz.debug("mySection is null");
        }

        List<QuestionResponse> responsesByStage = new ArrayList<>();
        StudyStatus status = null;
        logz.debug("Begin study status collection");
        for (StudyStatus ss : s.getStudyStatusCollection()) {
            responsesByStage.addAll(ss.getQuestionResponseCollection());
            if (ss.getStageCounter() == lastStage) {
                status = ss;
                logz.debug("Inside study status loop - study status found");
                break;
            }
        }
        if (status == null) {
            logz.debug("Status is null");
            throw ServiceException.notFound();
        }

        logz.debug("ArrayList: responsesByStage Size = " + responsesByStage.size());

        HashMap<QuestionResponse.Key, QuestionResponse> rollup = QuestionResponse.rollup(responsesByStage);
        logz.debug("Rollup size = " + rollup.size());

        //Stack<String> responses = new Stack<>();
        LinkedList<DocXFormGenerator.Response> responses = new LinkedList<>();

        logz.debug("Before Form Question Loop");
        logz.debug("Is mySection Null? : " + (mySection == null));

        if (mySection != null) {
            logz.debug("Section Narrative = " + mySection.getNarrative());
            logz.debug("Section Title = " + mySection.getTitle());
            logz.debug("Section Form Collection size = " + mySection.getContainedFormCollection().size());
            logz.debug("Section Form Question Collection size = " + mySection.getFormQuestionCollection().size());
        }

        logz.debug("Outside form question loop - begin");
        ArrayList<Integer> qrAnswersIDs = new ArrayList<Integer>();
        HashMap<Integer, Integer> dependency = new HashMap<Integer, Integer>();

        if (mySection != null)
            for (FormQuestion q : mySection.getFormQuestionCollection()) {

                QuestionResponse qrtemp = new QuestionResponse();
                qrtemp.setQuestionsID(q.getQuestion().getQuestionsID());

                QuestionResponse qr = rollup.get(new QuestionResponse.Key(qrtemp));
                if (qr == null) {
                    continue;
                }
                if (qr.getAnswersID() != null)
                    qrAnswersIDs.add(qr.getAnswersID());

                if (q.getRequiredPredecessorAnswersID() != null) {
                    if (qr.getAnswersID() != null) {
                        dependency.put(qr.getAnswersID(), q.getRequiredPredecessorAnswersID());
                    }
                    //				else if(qr.getAnswersID() == null){
                    //					dependency.put(q.getQuestion().getQuestionsID(), q.getRequiredPredecessorAnswersID());
                    //				}
                }
            }
        logz.debug("These are the answer id's");
        logz.debug(qrAnswersIDs.toString());

        if (mySection != null)
            for (FormQuestion q : mySection.getFormQuestionCollection()) {

                QuestionResponse qrtemp = new QuestionResponse();
                qrtemp.setQuestionsID(q.getQuestion().getQuestionsID());

                QuestionResponse qr = rollup.get(new QuestionResponse.Key(qrtemp));

                if (q.getRequiredPredecessorAnswersID() == null) { //|| (!q.getRequiredPredecessorAnswersID().toString().trim().equals("")) )){ //don't want to answer questions that don't have predecessor
                    //|| rollup.containsKey(q.getRequiredPredecessorAnswersID())
                    //does not like (!q.getRequiredPredecessorAnswersID().toString().trim().equals("") which looks like it says if it is the equivalent of null
                    //(q.getRequiredPredecessorAnswersID() != null) if it has a parent

                    logz.debug("No parent!");
                    if (qr == null) {
                        //            	logz.debug("Question response is [null]");
                        //throw ServiceException.notFound();
                        DocXFormGenerator.ResponseType type;
                        final String name = q.getQuestion().getResponseFormat().getName();
                        if (name.equals("radio"))
                            type = DocXFormGenerator.ResponseType.Boolean;
                        else
                            type = DocXFormGenerator.ResponseType.String;
                        responses.add(new DocXFormGenerator.Response(type, null));
                        continue;
                    }

                    //logz.debug("Question ID = " + q.getQuestion().getQuestionsID() + " :  Question = " + q.getQuestion().getQuestionHyperText()+  " : User response = " + qr.getResponse());

                    if (q.getQuestion().getResponseFormat().getName().equals("radio")) {
                        if (qr.getAnswersID() == null) {
                            //logz.debug("Question response answers is null");
                            responses.add(new DocXFormGenerator.Response(DocXFormGenerator.ResponseType.Boolean, null));
                            //throw ServiceException.notFound();
                        } else {
                            //fix in to compare primatives    
                            boolean response = qr.getAnswersID().intValue() == q.getQuestion().getAnswerCollection().iterator().next().getAnswersID().intValue();
                            responses.add(new DocXFormGenerator.Response(DocXFormGenerator.ResponseType.Boolean, String.valueOf(response)));
                        }
                    } else { // string
                        String response = qr.getResponse();
                        /*if (response == null)
                            throw ServiceException.notFound();*/
                        responses.add(new DocXFormGenerator.Response(DocXFormGenerator.ResponseType.String, response));
                    }
                } else if (q.getRequiredPredecessorAnswersID() != null) {
                    logz.debug("At question: " + q.getQuestion().getQuestionsID());
                    //if(qrAnswersIDs.contains(q.getRequiredPredecessorAnswersID()))
                    if (checkParent(q.getRequiredPredecessorAnswersID(), qrAnswersIDs, dependency)) {
                        if (q.getQuestion().getResponseFormat().getName().equals("radio")) {

                            boolean response = qr.getAnswersID().intValue() == q.getQuestion().getAnswerCollection().iterator().next().getAnswersID().intValue();
                            responses.add(new DocXFormGenerator.Response(DocXFormGenerator.ResponseType.Boolean, String.valueOf(response)));
                        } else {
                            String response = qr.getResponse();
                            responses.add(new DocXFormGenerator.Response(DocXFormGenerator.ResponseType.String, response));
                        }
                    } else {//inserting place holders for the docx template at the 2nd level dependency
                        if (q.getQuestion().getResponseFormat().getName().equals("radio")) {
                            responses.add(new DocXFormGenerator.Response(DocXFormGenerator.ResponseType.Boolean, null));
                        } else {
                            responses.add(new DocXFormGenerator.Response(DocXFormGenerator.ResponseType.String, " "));
                        }
                    }
                } else {//inserting place holders for the docx template
                    if (q.getQuestion().getResponseFormat().getName().equals("radio")) {
                        responses.add(new DocXFormGenerator.Response(DocXFormGenerator.ResponseType.Boolean, null));
                    } else {
                        responses.add(new DocXFormGenerator.Response(DocXFormGenerator.ResponseType.String, " "));
                    }
                    //logz.debug("I have a parent!");
                }
            }

        String wordDocumentType = "InformedConsent";
        if (sectionNumber == hipaaSectionNumber) { //consent form
            wordDocumentType = "HIPAA";
        }

        HashMap<Integer, Boolean> hm = new HashMap<Integer, Boolean>();
        ArrayList<Integer> qal = new ArrayList<Integer>();

        DocXFormGenerator docXFormGen = new DocXFormGenerator();
        //File file = new File("/usr/tomcat/cache/temp.docx");
        File file = new File("");
        try {
            file = docXFormGen.doGeneration(sectionNumber, responses, wordDocumentType, u, approved);
        } catch (Exception e) {
            logz.debug("Generating DocX", e);
            return Response.serverError().build();
        }

        final String filename = (approved ? "Approved " + wordDocumentType + " (Stage " + status.getStageCounter() + ", " + status.getStudyState().getName().trim() + ")" : wordDocumentType) + ".docx";
        //return Response.ok(file).header("Content-Disposition", "attachment; filename=\""+filename+'"').build();
        return Response.seeOther(sp.uploadStudyTeamFile(s, file, filename)).build();
    }

    private static boolean checkParent(int parentAnswerID, ArrayList<Integer> qrAnswersIDs, HashMap<Integer, Integer> dependency) {
        //module for checking 2nd level dependencies
        logz.debug("checking parent: " + parentAnswerID);

        if (!qrAnswersIDs.contains(parentAnswerID)) {//if you are not in the rollup
            //logz.debug("parent " + parentAnswerID + " isn't checked");
            return false;
        } else if (dependency.containsKey(parentAnswerID)) {//if you have a parent who has a grandparent
            //logz.debug("parent + " + parentAnswerID + " is checked and has a grandparent + " + dependency.get(parentAnswerID));
            return checkParent(dependency.get(parentAnswerID), qrAnswersIDs, dependency);
        }

        //logz.debug("parent " + parentAnswerID + " is checked");
        return true;
    }
}
