package RAMS.RAMSPasswordHelper;

import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.StandardCharsets;
import org.jasypt.encryption.pbe.StandardPBEStringEncryptor;


import org.apache.commons.codec.binary.Base64;
//import java.util.Base64;

public class PasswordHelper {
	private static final String ENCODE = "ENCODE";
	private static final String DECODE = "DECODE";
	private static final String ENCRYPT = "ENCRYPT";
	private static final String DECRYPT = "DECRYPT";
    private static final String PBEWITHMD5ANDTRIPLEDES = "PBEWithMD5AndTripleDES";
    		
	public static void main(String[] args) {
        String userName = "";
        String password = "";
        String action = "";
        int errors = 0;

        /*
         * Determine if we have the 3 parameters and validate them. 
         * The first parameter is action, which can be ENCODE or DECODE.
         * The second parameter is the userid in domain\\userid format.
         * The third parameter is the unencoded password the action is 
         * encode or the encoded password if the action is decode. 
         * */
        if (args.length == 3) {
      		action = args[0].toUpperCase();
      		if (action.equals(ENCODE) || action.equals(DECODE) || action.equals(ENCRYPT) || action.equals(DECRYPT)) {
      			System.out.println("action: " + action);
      		} else {
      			System.out.println("ERROR: Unknown action: " + action);
      			errors+=1;
      		}
      		if (errors == 0) {
      			if (action.equals(ENCODE)) {
      	      		if (args[1].length() > 0) {
      	      			userName = args[1];
      	      		} else {
      	      			System.out.println("ERROR: Missing userName ");
      	      			errors+=1;
      	      		}
      	      		if (args[2].length() > 0) {
      	      			password = args[2];
      	      		} else {
      	      			System.out.println("ERROR: Missing password ");
      	      			errors+=1;
      	      		}
      			} else if(action.equals(DECODE)){
      	      		if (args[1].length() > 0) {
      	      			userName = args[1];
      	      		} else {
      	      			System.out.println("ERROR: Missing userName ");
      	      			errors+=1;
      	      		}
      	      		if (args[2].length() > 0) {
      	      			password = args[2];
      	      		} else {
      	      			System.out.println("ERROR: Missing encoded password: ");
      	      			errors+=1;
      	      		}
     			} else if(action.equals(ENCRYPT)){
      	      		if (args[1].length() > 0) {
      	      			userName = args[1];
      	      		} else {
      	      			System.out.println("ERROR: Missing userName ");
      	      			errors+=1;
      	      		}
      	      		if (args[2].length() > 0) {
      	      			password = args[2];
      	      		} else {
      	      			System.out.println("ERROR: Missing password: ");
      	      			errors+=1;
      	      		}
     			} else if(action.equals(DECRYPT)){
          	      	if (args[1].length() > 0) {
          	      		userName = args[1];
          	      	} else {
          	      		System.out.println("ERROR: Missing userName ");
          	      		errors+=1;
          	      	}
          	      	if (args[2].length() > 0) {
          	      		password = args[2];
          	      	} else {
          	      		System.out.println("ERROR: Missing encrypted password: ");
          	      		errors+=1;
          	      	}
     			}
      		}
      	} else {
      		// Args.Length != 3, so show help.
      		System.out.println("ERROR: expected 3 parameters [ENCODE|DECODE|ENCRYPT|DECRYPT] userName [password|encodedOrEncryptedPassword]");
      		System.out.println("This PasswordHelper is used to encode passwords for use in the configuration XML files. It can be run as follows: " );
      		System.out.println("java -cp RAMSPasswordHelper-2.1.0-jar-with-dependencies.jar  RAMS.RAMSPasswordHelper.PasswordHelper ENCODE userName unEncodedPassword");
      		System.out.println("java -cp RAMSPasswordHelper-2.1.0-jar-with-dependencies.jar  RAMS.RAMSPasswordHelper.PasswordHelper DECODE userName encodedPassword");
      		System.out.println("java -cp RAMSPasswordHelper-2.1.0-jar-with-dependencies.jar  RAMS.RAMSPasswordHelper.PasswordHelper ENCRYPT userName unEncryptedPassword");
      		System.out.println("java -cp RAMSPasswordHelper-2.1.0-jar-with-dependencies.jar  RAMS.RAMSPasswordHelper.PasswordHelper DECRYPT userName encryptedPassword");
      	}
    	      	 
    	/*
    	 * Do the encoding or decoding
    	 */
        if (errors == 0) {
    		if (action.equals(ENCODE)) {
	    		System.out.println("Encoding username + password: " + userName + " " + password);
	    		char[] ep = encodePassword(userName, password);
	    		System.out.println("encoded password: " + new String(ep));
	    	} else if (action.equals(DECODE)) {
	    		System.out.println("Decoding username + encoded password: " + userName + " " + password);
	    		char[] pwd = decodePassword(userName, password);
	    		System.out.print("password: ");
	    		for (int i = 0; i < pwd.length; i++) {
					System.out.print(pwd[i]);
				}
	    		System.out.println();
	    	} else if (action.equals(ENCRYPT)) {
	    		System.out.println("Encrypting username + password: " + userName + " " + password);
	    		char[] pwd = encryptPassword(userName, password);
	    		System.out.print("password: ");
	    		for (int i = 0; i < pwd.length; i++) {
					System.out.print(pwd[i]);
				}
	    		System.out.println();
	    	} else if (action.equals(DECRYPT)) {
	    		System.out.println("Decrypting username + encrypted password: " + userName + " " + password);
	    		char[] pwd = decryptPassword(userName, password);
	    		System.out.print("password: ");
	    		for (int i = 0; i < pwd.length; i++) {
					System.out.print(pwd[i]);
				}
	    		System.out.println();
	    	}
    	}  	
        
    	return;
	}

	public static char[] encodePassword(String userName, String password) {
		byte[] encodedSalt = Base64.encodeBase64(userName.getBytes());
		byte[] encodedPassword = Base64.encodeBase64(password.getBytes());
		byte[] ep = new byte[encodedSalt.length + encodedPassword.length];
		System.arraycopy(encodedSalt, 0, ep, 0, encodedSalt.length);
		System.arraycopy(encodedPassword, 0, ep, encodedSalt.length, encodedPassword.length);
		char[] pwd = new String(ep).toCharArray();
		return pwd;
	}

	public static char[] decodePassword(String userName, String encodedPassword) {

		byte[] passwordBytes = Base64
				.decodeBase64(encodedPassword.substring(Base64.encodeBase64(userName.getBytes()).length).getBytes());
		CharBuffer cBuffer = StandardCharsets.UTF_8.decode(ByteBuffer.wrap(passwordBytes));

		char[] pwd = null;
		if (cBuffer.hasArray()) {
			pwd = new char[cBuffer.length()];
			pwd = cBuffer.array();
		} else {
			pwd = new char[cBuffer.length()];
			for (int p = 0; p < cBuffer.length(); p++) {
				pwd[p] = cBuffer.get(p);
			}
		}
		return pwd;

	}

	public static char[] encryptPassword(String userName, String password) {

		StandardPBEStringEncryptor encryptor = new StandardPBEStringEncryptor();
		encryptor.setPassword(userName); // we HAVE TO set a password
		encryptor.setAlgorithm(PBEWITHMD5ANDTRIPLEDES); // optionally set the
															// algorithm

		char[] pwd = encryptor.encrypt(password).toCharArray();

		return pwd;

	}

	public static char[] decryptPassword(String userName, String encryptedPassword) {

		StandardPBEStringEncryptor encryptor = new StandardPBEStringEncryptor();
		encryptor.setPassword(userName); // we HAVE TO set a password
		encryptor.setAlgorithm(PBEWITHMD5ANDTRIPLEDES); // optionally set the algorithm

		char[] pwd = encryptor.decrypt(encryptedPassword).toCharArray();

		return pwd;

	}
}
